import Vue from "vue";
import App from "@/App.vue";
import router from "@/router";
import ElementUI from "element-ui";
import "@/styles/index.scss";
import VueI18n from "vue-i18n";
import mixinApp from "@/mixins/app";
import { messages } from "@/i18n";
import "babel-polyfill";
import "@/libs/resize";
import { post, get, del, put } from "@/plugins/requests/index.js";
import api from "@/api";
import store from "@/store";
import Setting from "@/setting";

// 插件
import plugins from "@/plugins";
import filters from "@/plugins/filters";

Vue.use(plugins);

Object.keys(filters).forEach(item => Vue.filter(item, filters[item]));

Vue.prototype.api = api;
Vue.prototype.$get = get;
Vue.prototype.$post = post;
Vue.prototype.$del = del;
Vue.prototype.$put = put;

Vue.config.productionTip = false;
Vue.use(VueI18n);
Vue.use(ElementUI, { size: "small" });
const i18n = new VueI18n({
    locale: Setting.i18n.default,
    messages
});

Vue.prototype.auth = function(values){
    // 直接拿值进行匹配,取得当前router的mate信息,传入，返回一个boolean，以此进行vif判断
    // router.meta 信息匹配传进来的value，用vif去进行按钮级权限配置即可。
    // $router.currentRoute.meta
    // console.log(router.currentRoute.meta,'当前路由的按钮权限数组')
    return router.currentRoute.meta.btn.some(e=>e===values)
}
new Vue({
    mixins: [mixinApp],
    router,
    i18n,
    store,
    render: h => h(App)
}).$mount("#app");
